﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Analysis.SolutionCaching.PermutationEncoding {
  [Item("PermutationMutationPointcut", "Wraps a mutator and applies after-mutation actions.")]
  [StorableClass]
  public class PermutationMutationPointcut : MutationPointcut<IPermutationAdvice, IPermutationManipulator>, IPermutationManipulator, IPermutationPointcut {
    public ILookupParameter<Permutation> PermutationParameter {
      get { return (ILookupParameter<Permutation>)Parameters["Permutation"]; }
    }

    [StorableConstructor]
    protected PermutationMutationPointcut(bool deserializing) : base(deserializing) { }
    protected PermutationMutationPointcut(PermutationMutationPointcut original, Cloner cloner) : base(original, cloner) { }

    public PermutationMutationPointcut()
      : base() {
      Parameters.Add(new LookupParameter<Permutation>("Permutation", "The permutation that is being manipulated."));

      foreach (Type t in ApplicationManager.Manager.GetTypes(typeof(IPermutationManipulator))) {
        if ((!typeof(PermutationMutationPointcut).IsAssignableFrom(t)) &&
          (!typeof(MultiOperator<IPermutationManipulator>).IsAssignableFrom(t))) {
          MutatorsParameter.ValidValues.Add((IPermutationManipulator)Activator.CreateInstance(t));
        }
      }

      ParameterizeManipulators();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new PermutationMutationPointcut(this, cloner);
    }

    private void ParameterizeManipulators() {
      foreach (IPermutationManipulator manipulator in MutatorsParameter.ValidValues.OfType<IPermutationManipulator>()) {
        manipulator.PermutationParameter.ActualName = PermutationParameter.Name;
      }
      foreach (IStochasticOperator op in MutatorsParameter.ValidValues.OfType<IStochasticOperator>()) {
        op.RandomParameter.ActualName = RandomParameter.Name;
      }
    }
  }
}
